@@ -76,16 +76,19 @@ struct FMDirectoryViewClass {
 	 * It must be replaced by each subclass.
 	 */
 	void    (* add_file) 		 (FMDirectoryView *view, 
-					  NautilusFile *file);
+					  NautilusFile *file,
+					  NautilusDirectory *directory);
 	void    (* remove_file)		 (FMDirectoryView *view, 
-					  NautilusFile *file);
+					  NautilusFile *file,
+					  NautilusDirectory *directory);
 
 	/* The 'file_changed' signal is emitted to signal a change in a file,
 	 * including the file being removed.
 	 * It must be replaced by each subclass.
 	 */
 	void 	(* file_changed)         (FMDirectoryView *view, 
-					  NautilusFile *file);
+					  NautilusFile *file,
+					  NautilusDirectory *directory);
 
 	/* The 'end_file_changes' signal is emitted after a set of files
 	 * are added to the view. It can be replaced by a subclass to do any 
@@ -118,7 +121,8 @@ struct FMDirectoryViewClass {
 	 * load failures like ACCESS_DENIED.
 	 */
 	void    (* load_error)           (FMDirectoryView *view,
-					  GnomeVFSResult result);
+					  GnomeVFSResult result,
+					  const char *error_message);
 
 	/* Function pointers that don't have corresponding signals */
 
@@ -213,8 +217,9 @@ struct FMDirectoryViewClass {
 	 * to provide a sorting order to determine which files should be
 	 * presented when only a partial list is provided.
 	 */
-	void     (* sort_files)                 (FMDirectoryView *view,
-						 GList          **files);
+	int     (* compare_files)              (FMDirectoryView *view,
+						NautilusFile    *a,
+						NautilusFile    *b);
 
 	/* get_emblem_names_to_exclude is a function pointer that subclasses
 	 * may override to specify a set of emblem names that should not
@@ -246,6 +251,13 @@ struct FMDirectoryViewClass {
 	 */
 	gboolean (* supports_zooming)		(FMDirectoryView *view);
 
+	/* using_manual_layout is a function pointer that subclasses may
+	 * override to control whether or not items can be freely positioned
+	 * on the user-visible area.
+	 * Note that this value is not guaranteed to be constant within the
+	 * view's lifecycle. */
+	gboolean (* using_manual_layout)     (FMDirectoryView *view);
+
 	/* is_read_only is a function pointer that subclasses may
 	 * override to control whether or not the user is allowed to
 	 * change the contents of the currently viewed directory. The
@@ -283,7 +295,8 @@ struct FMDirectoryViewClass {
 						 gboolean select_all);
 
 	gboolean (* file_still_belongs)		(FMDirectoryView *view,
-						 NautilusFile	 *file);
+						 NautilusFile	 *file,
+						 NautilusDirectory *directory);
 
 	/* Preference change callbacks, overriden by icon and list views. 
 	 * Icon and list views respond by synchronizing to the new preference
@@ -338,6 +351,7 @@ gboolean            fm_directory_view_supports_creating_files          (FMDirect
 gboolean            fm_directory_view_accepts_dragged_files            (FMDirectoryView  *view);
 gboolean            fm_directory_view_supports_properties              (FMDirectoryView  *view);
 gboolean            fm_directory_view_supports_zooming                 (FMDirectoryView  *view);
+gboolean            fm_directory_view_using_manual_layout              (FMDirectoryView  *view);
 void                fm_directory_view_move_copy_items                  (const GList      *item_uris,
 									GArray           *relative_item_points,
 									const char       *target_uri,
@@ -354,6 +368,8 @@ void                fm_directory_view_clear                            (FMDirect
 void                fm_directory_view_begin_loading                    (FMDirectoryView  *view);
 void                fm_directory_view_end_loading                      (FMDirectoryView  *view);
 
+gboolean            fm_directory_view_get_loading                      (FMDirectoryView  *view);
+
 /* Hooks for subclasses to call. These are normally called only by 
  * FMDirectoryView and its subclasses 
  */
@@ -374,6 +390,7 @@ NautilusDirectory  *fm_directory_view_get_model                        (FMDirect
 GtkWindow	   *fm_directory_view_get_containing_window	       (FMDirectoryView  *view);
 NautilusFile       *fm_directory_view_get_directory_as_file            (FMDirectoryView  *view);
 EelBackground *     fm_directory_view_get_background                   (FMDirectoryView  *view);
+gboolean            fm_directory_view_get_allow_moves                  (FMDirectoryView  *view);
 void                fm_directory_view_pop_up_background_context_menu   (FMDirectoryView  *view,
 									GdkEventButton   *event);
 void                fm_directory_view_pop_up_selection_context_menu    (FMDirectoryView  *view,
@@ -387,21 +404,25 @@ gboolean	    fm_directory_view_should_sort_directories_first    (FMDirectoryView
 void                fm_directory_view_update_menus                     (FMDirectoryView  *view);
 void                fm_directory_view_new_folder                       (FMDirectoryView  *view);
 void                fm_directory_view_new_file                         (FMDirectoryView  *view,
+									const char       *parent_uri,
 									NautilusFile     *source);
 void                fm_directory_view_ignore_hidden_file_preferences   (FMDirectoryView  *view);
 void                fm_directory_view_init_view_iface                  (NautilusViewIface *iface);
 void                fm_directory_view_handle_url_drop                  (FMDirectoryView  *view,
 									const char       *encoded_url,
+									const char       *target_uri,
 									GdkDragAction     action,
 									int               x,
 									int               y);
 void                fm_directory_view_handle_uri_list_drop             (FMDirectoryView  *view,
 									const char       *item_uris,
+									const char       *target_uri,
 									GdkDragAction     action,
 									int               x,
 									int               y);
 void                fm_directory_view_handle_text_drop                 (FMDirectoryView  *view,
 									const char       *text,
+									const char       *target_uri,
 									GdkDragAction     action,
 									int               x,
 									int               y);
@@ -412,4 +433,6 @@ void                fm_directory_view_add_subdirectory                (FMDirecto
 void                fm_directory_view_remove_subdirectory             (FMDirectoryView  *view,
 									NautilusDirectory*directory);
 
+gboolean            fm_directory_view_is_editable                     (FMDirectoryView *view);
+
 #endif /* FM_DIRECTORY_VIEW_H */
